/*
codeur.h - Library for encoder arduino conected
Version 1.1

by Cordier Yves december 2016 Version du 07/12/2016
release :
V1.0 : 07/12/2016
V1.1 : 10/12/2016 ; ajout de la prise en compte de la piste Z et des fonctions associées
V1.2 : 29/12/2016 ; amélioration général du code
v1.21 : 14/06/2017 ; correction mineure declarations volatile des compteurs
v1.22 : 19/06/2017 ; rajout du sens de comptage pour corriger les mauvais branchements eventuels
					 rajout de la méthode setDirection(boolean mode)
v1.3  : 28/07/2020 ; utilisation de la bibliotheque d'entree/sortie Rapide FastInOut
					 utilisation d'une moyenne exponentielle moins gourmande en calcul pour le 
					 calcul de la vitesse moyenne


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef codeur_h
#define codeur_h

// utilisé pour les tests avec une sortie sur oscilloscope
// à mettre en commentaire dans version finalisée
// sur un avr, on pilote la broche N°8 pour matérialiser le début et fin d'interruption
// PortB masque 0x01 (bit de poids 0)
//#define _testOscillo

// acceleration via la bibliotheque FastInOut
#define _ESrapide 
// *****************************************

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif
// La bibliotheque d'entree sortie rapide
#ifdef _ESrapide
#include <FastInOut.h>
#endif

// ******************************************************
// Les diverses constantes définssant le type de comptage
// ******************************************************
// Celles à adapter au cas utilisateur
#define maxCodeur 3 // 6 codeurs au maximum pour cette bibliotheque
#define zerotrackexist // prise en compte de la piste zero dans la bibliotheque
// ********************************************************
// Les autres inamovibles
#define bidirect	0x02
#define unidirect	0x00

#define doubleprecision		0x01
#define simpleprecision		0x00
#define singleprecision		(simpleprecision)

#define bidirdouble (bidirect)+(doubleprecision)
#define unidirdouble (unidirect)+(doubleprecision)
#define bidirsimple (bidirect)+(simpleprecision)
#define unidirsimple (unidirect)+(simpleprecision)
#define bidirsingle (bidirsimple)
#define unidirsingle (unidirsimple)

#define pulsezerotrack     0x04
#define pistetopzero     0x04

#define avgspeedcompute 0x40
#define calculvitessemoy 0x40
#define speedcompute 0x80
#define calculvitesse 0x80

// Les codes erreurs
#define pinpulsenotaninterrupt 0x01
#define notanencoderavailable 0x04
#define bidirectimpossible    0x08
#define pinzeronotaninterrupt 0x02

// pour pallier à une erreur sur NRF5 implementation
#ifndef NOT_AN_INTERRUPT
#define NOT_AN_INTERRUPT -1 
#endif

// **************************************
// L'entête de la classe
// **************************************
#define maxtablevitesse 10
class encoder
{	
public :

private:
	long volatile compt; // le compteur courant
	int increment; // l'increment de comptage +1 ou -1 suivant le sens de comptage
	int vit; // la vitesse en nb pulse par seconde
	int vitMoy; // la vitesse moyenne glissante sur 10 pulses
	unsigned long dernierPulse; // util pour calculer la vitesse
	// voie A pin interruptible
#ifdef _ESrapide
	FastInOut *brochePinPulse=NULL; 
#else
	int pinPulse; 
#endif
	byte interpulse; // interruption pulse
	// voie B pin quelconque utilisé dans les modes bidir.. (<0 si non utilisée)
#ifdef _ESrapide
	FastInOut *brochePinSens=NULL; 
#else
	int pinSens; 
#endif
	boolean stoA; // Pour gérer les fronts de la voie A (qui génère l'interruption)
	boolean marche; // marche arret du compteur
	boolean sens; // détermination du sens (comptage ou décomptage) utilisé pour le calcul de la vitesse
	byte typeCodeur; // unidir.., bidir..; ..simpleprec, singleprec,  unidirect, bidirect, doubleprec ...;
	boolean vitCalcul; // calcul-on la vitesse ?
	boolean vitMoyCalcul; // calcul-on la vitesse moyenne ?
	byte numCodeur; // le numero du codeur
	int tablevitesse[maxtablevitesse];
	byte pointeurvitesse;
	byte nbvitesse;
	float alpha;
	float unMoinsAlpha;

#ifdef zerotrackexist
// voie Z pin interruptible (<0 si innutilisée)
#ifdef _ESrapide
	FastInOut *brochePinZero=NULL; 
#else
	int pinzero; 
#endif
	byte interzero; // interrruption voie zero
	boolean pomArmed; // est on en attente de pom
	boolean pisteZeroOK; // prend on en compte la piste zero
#endif

	byte numerreur; // quelle erreur ?
	
	static encoder* ancre0;
	static void interruption0();// La procedure d'interruption
	static void interruptionzero0();// La procedure d'interruption
	#if maxCodeur > 1
	static encoder* ancre1;
	static void interruption1();// La procedure d'interruption
	static void interruptionzero1();// La procedure d'interruption
	#endif
	#if maxCodeur > 2
	static encoder* ancre2;
	static void interruption2();// La procedure d'interruption
	static void interruptionzero2();// La procedure d'interruption
	#endif
	#if maxCodeur > 3
	static encoder* ancre3;
	static void interruption3();// La procedure d'interruption
	static void interruptionzero3();// La procedure d'interruption
	#endif
	#if maxCodeur > 4
	static encoder* ancre4;
	static void interruption4();// La procedure d'interruption
	static void interruptionzero4();// La procedure d'interruption
	#endif
	#if maxCodeur > 5
	static encoder* ancre5;
	static void interruption5();// La procedure d'interruption
	static void interruptionzero5();// La procedure d'interruption
	#endif
	
	
public:
	long val(); // renvoie le compteur courant
	int speed(); // renvoie la vitesse calculée 0 si non calculés
	int avgSpeed(); // renvoie la vitesse moyenne glissante en utilisant la moyenne logarithmique
	void setAlpha(float alph=0.35); // positionne le coefficient de moyenne exponentielle 0.25 par defaut
	void evolue(); // n'est jamais appelé par l'utilisateur mais correspond à la procédure appelée par l'interruption de comptage décomptage
	void reset(); // remise à zero
	void run(); // marche
	void stop(); // arret
	void setVal(long v); // impose la valeur courante du compteur
	void setDirection(boolean mode);  // Change le sens de comptage
#ifdef zerotrackexist
	void pomSet(); // preparation à la prise de pom
	void pomReset(); // annulation de l prise de pom
	void pom(); // n'est jamais appelé par l'utilisateur mais correspond à la procédure appelée par l'interruption Prise Origine Machine
	boolean pomDone(); // renvoi l'état des pom (si un pomSet() à été demandé, renvoi si oui ou non les poms ont été faites)
	// sinon renvoi true;
#endif
	byte errornum() ; //renvoi la combinaison des erreurs de création 0 si aucunes
	// Createur
	
#ifdef zerotrackexist
	encoder(byte type, int num1, int num2 = -1, int num3=-1 );  // les deux dernières valeur est optionelle dans le cas d'un codeur unidirectionnel
#else
	encoder(byte type, int num1, int num2 = -1); // La derniere valeur est optionelle dans le même cas
#endif 

	// destructeur
	~encoder();
};

#endif
